@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2010       Mehdi Juhoor
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2010  Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +19,8 @@
 #include "xlu.h"
 #include "xlu_kernels.h"
 
+#include "starpu_cusolver.h"
+
 /*
  *	Construct the DAG
  */
@@ -34,59 +36,61 @@ static struct starpu_task *create_task(starpu_tag_t id)
 	return task;
 }
 
-static struct starpu_task *create_task_11(starpu_data_handle_t dataA, unsigned k, unsigned no_prio)
+static struct starpu_task *create_task_getrf(starpu_data_handle_t dataA, unsigned k, unsigned no_prio, int nblocks)
 {
-/*	printf("task 11 k = %d TAG = %llx\n", k, (TAG11(k))); */
+/*	printf("task GETRF k = %d TAG = %llx\n", k, (TAG_GETRF(k))); */
 
-	struct starpu_task *task = create_task(TAG11(k));
+	struct starpu_task *task = create_task(TAG_GETRF(k));
 
-	task->cl = &cl11;
+	task->cl = &cl_getrf;
 	task->color = 0xffff00;
 
 	/* which sub-data is manipulated ? */
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_LIBCUSOLVER)
+	task->handles[1] = scratch;
+#endif
+
 	/* this is an important task */
 	if (!no_prio)
-		task->priority = STARPU_MAX_PRIO;
+		task->priority = 3*nblocks - 3*k; /* Bottom-level-based prio */
 
 	/* enforce dependencies ... */
 	if (k > 0)
 	{
-		starpu_tag_declare_deps(TAG11(k), 1, TAG22(k-1, k, k));
+		starpu_tag_declare_deps(TAG_GETRF(k), 1, TAG_GEMM(k-1, k, k));
 	}
 
 	return task;
 }
 
-static int create_task_12(starpu_data_handle_t dataA, unsigned k, unsigned j, unsigned no_prio)
+static int create_task_trsm_ll(starpu_data_handle_t dataA, unsigned k, unsigned j, unsigned no_prio, int nblocks)
 {
 	int ret;
 
-/*	printf("task 12 k,i = %d,%d TAG = %llx\n", k,i, TAG12(k,i)); */
+/*	printf("task TRSM_LL k,i = %d,%d TAG = %llx\n", k,i, TAG_TRSM_LL(k,i)); */
 
-	struct starpu_task *task = create_task(TAG12(k, j));
+	struct starpu_task *task = create_task(TAG_TRSM_LL(k, j));
 
-	task->cl = &cl12;
+	task->cl = &cl_trsm_ll;
 	task->color = 0x8080ff;
 
 	/* which sub-data is manipulated ? */
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 	task->handles[1] = starpu_data_get_sub_data(dataA, 2, j, k);
 
-	if (!no_prio && (j == k+1))
-	{
-		task->priority = STARPU_MAX_PRIO;
-	}
+	if (!no_prio)
+		task->priority = 3*nblocks - (2*k + j); /* Bottom-level-based prio */
 
 	/* enforce dependencies ... */
 	if (k > 0)
 	{
-		starpu_tag_declare_deps(TAG12(k, j), 2, TAG11(k), TAG22(k-1, k, j));
+		starpu_tag_declare_deps(TAG_TRSM_LL(k, j), 2, TAG_GETRF(k), TAG_GEMM(k-1, k, j));
 	}
 	else
 	{
-		starpu_tag_declare_deps(TAG12(k, j), 1, TAG11(k));
+		starpu_tag_declare_deps(TAG_TRSM_LL(k, j), 1, TAG_GETRF(k));
 	}
 
 	ret = starpu_task_submit(task);
@@ -94,31 +98,29 @@ static int create_task_12(starpu_data_handle_t dataA, unsigned k, unsigned j, un
 	return ret;
 }
 
-static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned i, unsigned no_prio)
+static int create_task_trsm_ru(starpu_data_handle_t dataA, unsigned k, unsigned i, unsigned no_prio, int nblocks)
 {
 	int ret;
-	struct starpu_task *task = create_task(TAG21(k, i));
+	struct starpu_task *task = create_task(TAG_TRSM_RU(k, i));
 
-	task->cl = &cl21;
+	task->cl = &cl_trsm_ru;
 	task->color = 0x8080c0;
 
 	/* which sub-data is manipulated ? */
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 	task->handles[1] = starpu_data_get_sub_data(dataA, 2, k, i);
 
-	if (!no_prio && (i == k+1))
-	{
-		task->priority = STARPU_MAX_PRIO;
-	}
+	if (!no_prio)
+		task->priority = 3*nblocks - (2*k + i); /* Bottom-level-based prio */
 
 	/* enforce dependencies ... */
 	if (k > 0)
 	{
-		starpu_tag_declare_deps(TAG21(k, i), 2, TAG11(k), TAG22(k-1, i, k));
+		starpu_tag_declare_deps(TAG_TRSM_RU(k, i), 2, TAG_GETRF(k), TAG_GEMM(k-1, i, k));
 	}
 	else
 	{
-		starpu_tag_declare_deps(TAG21(k, i), 1, TAG11(k));
+		starpu_tag_declare_deps(TAG_TRSM_RU(k, i), 1, TAG_GETRF(k));
 	}
 
 	ret = starpu_task_submit(task);
@@ -126,35 +128,33 @@ static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned i, un
 	return ret;
 }
 
-static int create_task_22(starpu_data_handle_t dataA, unsigned k, unsigned i, unsigned j, unsigned no_prio)
+static int create_task_gemm(starpu_data_handle_t dataA, unsigned k, unsigned i, unsigned j, unsigned no_prio, int nblocks)
 {
 	int ret;
 
-/*	printf("task 22 k,i,j = %d,%d,%d TAG = %llx\n", k,i,j, TAG22(k,i,j)); */
+/*	printf("task GEMM k,i,j = %d,%d,%d TAG = %llx\n", k,i,j, TAG_GEMM(k,i,j)); */
 
-	struct starpu_task *task = create_task(TAG22(k, i, j));
+	struct starpu_task *task = create_task(TAG_GEMM(k, i, j));
 
-	task->cl = &cl22;
+	task->cl = &cl_gemm;
 	task->color = 0x00ff00;
 
 	/* which sub-data is manipulated ? */
-	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, i); /* produced by TAG21(k, i) */
-	task->handles[1] = starpu_data_get_sub_data(dataA, 2, j, k); /* produced by TAG12(k, j) */
-	task->handles[2] = starpu_data_get_sub_data(dataA, 2, j, i); /* produced by TAG22(k-1, i, j) */
+	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, i); /* produced by TAG_TRSM_RU(k, i) */
+	task->handles[1] = starpu_data_get_sub_data(dataA, 2, j, k); /* produced by TAG_TRSM_LL(k, j) */
+	task->handles[2] = starpu_data_get_sub_data(dataA, 2, j, i); /* produced by TAG_GEMM(k-1, i, j) */
 
-	if (!no_prio &&  (i == k + 1) && (j == k +1) )
-	{
-		task->priority = STARPU_MAX_PRIO;
-	}
+	if (!no_prio)
+		task->priority = 3*nblocks - (k + i + j); /* Bottom-level-based prio */
 
 	/* enforce dependencies ... */
 	if (k > 0)
 	{
-		starpu_tag_declare_deps(TAG22(k, i, j), 3, TAG22(k-1, i, j), TAG12(k, j), TAG21(k, i));
+		starpu_tag_declare_deps(TAG_GEMM(k, i, j), 3, TAG_GEMM(k-1, i, j), TAG_TRSM_LL(k, j), TAG_TRSM_RU(k, i));
 	}
 	else
 	{
-		starpu_tag_declare_deps(TAG22(k, i, j), 2, TAG12(k, j), TAG21(k, i));
+		starpu_tag_declare_deps(TAG_GEMM(k, i, j), 2, TAG_TRSM_LL(k, j), TAG_TRSM_RU(k, i));
 	}
 
 	ret = starpu_task_submit(task);
@@ -183,7 +183,7 @@ static int dw_codelet_facto_v3(starpu_data_handle_t dataA, unsigned nblocks, uns
 	for (k = 0; k < nblocks; k++)
 	{
 		starpu_iteration_push(k);
-		struct starpu_task *task = create_task_11(dataA, k, no_prio);
+		struct starpu_task *task = create_task_getrf(dataA, k, no_prio, nblocks);
 
 		/* we defer the launch of the first task */
 		if (k == 0)
@@ -199,9 +199,9 @@ static int dw_codelet_facto_v3(starpu_data_handle_t dataA, unsigned nblocks, uns
 
 		for (i = k+1; i<nblocks; i++)
 		{
-			ret = create_task_12(dataA, k, i, no_prio);
+			ret = create_task_trsm_ll(dataA, k, i, no_prio, nblocks);
 			if (ret == -ENODEV) return ret;
-			ret = create_task_21(dataA, k, i, no_prio);
+			ret = create_task_trsm_ru(dataA, k, i, no_prio, nblocks);
 			if (ret == -ENODEV) return ret;
 		}
 
@@ -209,7 +209,7 @@ static int dw_codelet_facto_v3(starpu_data_handle_t dataA, unsigned nblocks, uns
 		{
 			for (j = k+1; j<nblocks; j++)
 			{
-			     ret = create_task_22(dataA, k, i, j, no_prio);
+				ret = create_task_gemm(dataA, k, i, j, no_prio, nblocks);
 			     if (ret == -ENODEV) return ret;
 			}
 		}
@@ -223,7 +223,7 @@ static int dw_codelet_facto_v3(starpu_data_handle_t dataA, unsigned nblocks, uns
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
 	/* stall the application until the end of computations */
-	starpu_tag_wait(TAG11(nblocks-1));
+	starpu_tag_wait(TAG_GETRF(nblocks-1));
 
 	end = starpu_timing_now();
 
@@ -275,8 +275,12 @@ int STARPU_LU(lu_decomposition)(TYPE *matA, unsigned size, unsigned ld, unsigned
 
 	starpu_data_map_filters(dataA, 2, &f, &f2);
 
+	lu_kernel_init(size / nblocks);
+
 	int ret = dw_codelet_facto_v3(dataA, nblocks, no_prio);
 
+	lu_kernel_fini();
+
 	/* gather all the data */
 	starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 	starpu_data_unregister(dataA);
