@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -95,7 +95,7 @@ static struct starpu_codelet minmax_redux_codelet =
 {
 	.cpu_funcs = {minmax_redux_cpu_func},
 	.cpu_funcs_name = {"minmax_redux_cpu_func"},
-	.modes = {STARPU_RW, STARPU_R},
+	.modes = {STARPU_RW|STARPU_COMMUTE, STARPU_R},
 	.nbuffers = 2,
 	.name = "redux"
 };
@@ -148,7 +148,7 @@ int main(void)
 	int ret;
 
 	/* Not supported yet */
-	if (starpu_get_env_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
+	if (starpu_getenv_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
 		return 77;
 
 	ret = starpu_init(NULL);
@@ -159,13 +159,13 @@ int main(void)
 	unsigned long nelems = _nblocks*_entries_per_bock;
 	size_t size = nelems*sizeof(TYPE);
 
-	_x = (TYPE *) malloc(size);
-	_x_handles = (starpu_data_handle_t *) calloc(_nblocks, sizeof(starpu_data_handle_t));
+	starpu_malloc((void **)&_x, size*sizeof(TYPE));
+	_x_handles = (starpu_data_handle_t *) malloc(_nblocks * sizeof(starpu_data_handle_t));
 
 	assert(_x && _x_handles);
 
 	/* Initialize the vector with random values */
-        starpu_srand48(0);
+	starpu_srand48(0);
 	for (i = 0; i < nelems; i++)
 		_x[i] = (TYPE)starpu_drand48();
 
@@ -217,7 +217,7 @@ int main(void)
 
 	STARPU_ASSERT(_minmax[0] <= _minmax[1]);
 
-	free(_x);
+	starpu_free_noflag(_x, size*sizeof(TYPE));
 	free(_x_handles);
 	starpu_shutdown();
 
