@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,7 +36,11 @@
 #include <xbt/synchro_core.h>
 #endif
 #include <smpi/smpi.h>
+#if !defined(HAVE_SG_ACTOR_GET_DATA) && !defined(HAVE_SG_ACTOR_DATA) && \
+	(defined(HAVE_SMPI_PROCESS_SET_USER_DATA) || defined(smpi_process_get_user_data)) && \
+	!(defined(HAVE_MSG_PROCESS_SELF_NAME) || defined(MSG_process_self_name))
 #include <simgrid/simix.h>
+#endif
 #else
 
 #if defined(STARPU_LINUX_SYS) && defined(STARPU_HAVE_XCHG)
@@ -57,8 +61,6 @@ static int _starpu_futex_wake = FUTEX_WAKE;
 
 #ifdef STARPU_SIMGRID
 
-extern int _starpu_simgrid_thread_start(int argc, char *argv[]);
-
 int starpu_pthread_equal(starpu_pthread_t t1, starpu_pthread_t t2)
 {
 	return t1 == t2;
@@ -76,9 +78,12 @@ starpu_pthread_t starpu_pthread_self(void)
 int starpu_pthread_create_on(const char *name, starpu_pthread_t *thread, const starpu_pthread_attr_t *attr STARPU_ATTRIBUTE_UNUSED, void *(*start_routine) (void *), void *arg, starpu_sg_host_t host)
 {
 	char **_args;
+	int ret;
 	_STARPU_MALLOC(_args, 3*sizeof(char*));
-	asprintf(&_args[0], "%p", start_routine);
-	asprintf(&_args[1], "%p", arg);
+	ret = asprintf(&_args[0], "%p", start_routine);
+	STARPU_ASSERT(ret);
+	ret = asprintf(&_args[1], "%p", arg);
+	STARPU_ASSERT(ret);
 	_args[2] = NULL;
 	if (!host)
 		host = _starpu_simgrid_get_host_by_name("MAIN");
@@ -86,8 +91,16 @@ int starpu_pthread_create_on(const char *name, starpu_pthread_t *thread, const s
 	void *tsd;
 	_STARPU_CALLOC(tsd, MAX_TSD+1, sizeof(void*));
 
+#ifndef HAVE_SG_ACTOR_SET_STACKSIZE
+	if (attr && attr->stacksize)
+		_starpu_simgrid_set_stack_size(attr->stacksize);
+#endif
 #ifdef HAVE_SG_ACTOR_INIT
 	*thread= sg_actor_init(name, host);
+#ifdef HAVE_SG_ACTOR_SET_STACKSIZE
+	if (attr && attr->stacksize)
+		sg_actor_set_stacksize(*thread, attr->stacksize);
+#endif
 #ifdef HAVE_SG_ACTOR_SET_DATA
 	sg_actor_set_data(*thread, tsd);
 #else
@@ -104,6 +117,10 @@ int starpu_pthread_create_on(const char *name, starpu_pthread_t *thread, const s
 #endif
 #endif
 #endif
+#ifndef HAVE_SG_ACTOR_SET_STACKSIZE
+	if (attr && attr->stacksize)
+		_starpu_simgrid_set_stack_size(_starpu_default_stack_size);
+#endif
 
 #if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
 #  ifdef HAVE_SG_ACTOR_REF
@@ -141,6 +158,18 @@ int starpu_pthread_join(starpu_pthread_t thread STARPU_ATTRIBUTE_UNUSED, void **
 	return 0;
 }
 
+int starpu_pthread_detach(starpu_pthread_t thread STARPU_ATTRIBUTE_UNUSED)
+{
+#if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
+#  ifdef HAVE_SG_ACTOR_REF
+	sg_actor_unref(thread);
+#  else
+	MSG_process_unref(thread);
+#  endif
+#endif
+	return 0;
+}
+
 int starpu_pthread_exit(void *retval STARPU_ATTRIBUTE_UNUSED)
 {
 #ifdef HAVE_SG_ACTOR_SELF
@@ -152,8 +181,9 @@ int starpu_pthread_exit(void *retval STARPU_ATTRIBUTE_UNUSED)
 }
 
 
-int starpu_pthread_attr_init(starpu_pthread_attr_t *attr STARPU_ATTRIBUTE_UNUSED)
+int starpu_pthread_attr_init(starpu_pthread_attr_t *attr)
 {
+	attr->stacksize = 0;
 	return 0;
 }
 
@@ -162,6 +192,12 @@ int starpu_pthread_attr_destroy(starpu_pthread_attr_t *attr STARPU_ATTRIBUTE_UNU
 	return 0;
 }
 
+int starpu_pthread_attr_setstacksize(starpu_pthread_attr_t *attr, size_t stacksize)
+{
+	attr->stacksize = stacksize;
+	return 0;
+}
+
 int starpu_pthread_attr_setdetachstate(starpu_pthread_attr_t *attr STARPU_ATTRIBUTE_UNUSED, int detachstate STARPU_ATTRIBUTE_UNUSED)
 {
 	return 0;
@@ -486,7 +522,7 @@ int starpu_pthread_cond_destroy(starpu_pthread_cond_t *cond)
 }
 
 /* TODO: use rwlocks
- * https://gforge.inria.fr/tracker/index.php?func=detail&aid=17213&group_id=12&atid=165
+ * https://framagit.org/simgrid/simgrid/-/issues/92
  */
 int starpu_pthread_rwlock_init(starpu_pthread_rwlock_t *restrict rwlock, const starpu_pthread_rwlockattr_t *restrict attr STARPU_ATTRIBUTE_UNUSED)
 {
@@ -502,7 +538,7 @@ int starpu_pthread_rwlock_rdlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_RDLOCKING_RWLOCK();
 
- 	int p_ret = starpu_pthread_mutex_lock(rwlock);
+	int p_ret = starpu_pthread_mutex_lock(rwlock);
 
 	_STARPU_TRACE_RWLOCK_RDLOCKED();
 
@@ -523,7 +559,7 @@ int starpu_pthread_rwlock_wrlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_WRLOCKING_RWLOCK();
 
- 	int p_ret = starpu_pthread_mutex_lock(rwlock);
+	int p_ret = starpu_pthread_mutex_lock(rwlock);
 
 	_STARPU_TRACE_RWLOCK_WRLOCKED();
 
@@ -545,7 +581,7 @@ int starpu_pthread_rwlock_unlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_UNLOCKING_RWLOCK();
 
- 	int p_ret = starpu_pthread_mutex_unlock(rwlock);
+	int p_ret = starpu_pthread_mutex_unlock(rwlock);
 
 	_STARPU_TRACE_RWLOCK_UNLOCKED();
 
@@ -843,7 +879,7 @@ int starpu_pthread_cond_wait(starpu_pthread_cond_t *cond, starpu_pthread_mutex_t
 {
 	_STARPU_TRACE_COND_WAIT_BEGIN();
 
- 	int p_ret = pthread_cond_wait(cond, mutex);
+	int p_ret = pthread_cond_wait(cond, mutex);
 
 	_STARPU_TRACE_COND_WAIT_END();
 
@@ -854,7 +890,7 @@ int starpu_pthread_rwlock_rdlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_RDLOCKING_RWLOCK();
 
- 	int p_ret = pthread_rwlock_rdlock(rwlock);
+	int p_ret = pthread_rwlock_rdlock(rwlock);
 
 	_STARPU_TRACE_RWLOCK_RDLOCKED();
 
@@ -865,7 +901,7 @@ int starpu_pthread_rwlock_tryrdlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_RDLOCKING_RWLOCK();
 
- 	int p_ret = pthread_rwlock_tryrdlock(rwlock);
+	int p_ret = pthread_rwlock_tryrdlock(rwlock);
 
 	if (!p_ret)
 		_STARPU_TRACE_RWLOCK_RDLOCKED();
@@ -877,7 +913,7 @@ int starpu_pthread_rwlock_wrlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_WRLOCKING_RWLOCK();
 
- 	int p_ret = pthread_rwlock_wrlock(rwlock);
+	int p_ret = pthread_rwlock_wrlock(rwlock);
 
 	_STARPU_TRACE_RWLOCK_WRLOCKED();
 
@@ -888,7 +924,7 @@ int starpu_pthread_rwlock_trywrlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_WRLOCKING_RWLOCK();
 
- 	int p_ret = pthread_rwlock_trywrlock(rwlock);
+	int p_ret = pthread_rwlock_trywrlock(rwlock);
 
 	if (!p_ret)
 		_STARPU_TRACE_RWLOCK_WRLOCKED();
@@ -900,7 +936,7 @@ int starpu_pthread_rwlock_unlock(starpu_pthread_rwlock_t *rwlock)
 {
 	_STARPU_TRACE_UNLOCKING_RWLOCK();
 
- 	int p_ret = pthread_rwlock_unlock(rwlock);
+	int p_ret = pthread_rwlock_unlock(rwlock);
 
 	_STARPU_TRACE_RWLOCK_UNLOCKED();
 
