@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2021-2021  Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -98,6 +99,7 @@ static int _starpu_arbiter_filter_modes(int mode)
 	mode &= ~STARPU_COMMUTE;
 	mode &= ~STARPU_SSEND;
 	mode &= ~STARPU_LOCALITY;
+	mode &= ~STARPU_NOFOOTPRINT;
 	if (mode == STARPU_RW)
 		mode = STARPU_W;
 	return mode;
@@ -136,8 +138,8 @@ struct LockOrDelegateListNode
 	struct LockOrDelegateListNode* next;
 };
 
-/* Post a task to perfom if possible, otherwise put it in the list
- * If we can perfom this task, we may also perfom all the tasks in the list
+/* Post a task to perform if possible, otherwise put it in the list
+ * If we can perform this task, we may also perform all the tasks in the list
  * This function return 1 if the task (and maybe some others) has been done
  * by the calling thread and 0 otherwise (if the task has just been put in the list)
  */
@@ -203,7 +205,7 @@ static int _starpu_LockOrDelegatePostOrPerform(starpu_arbiter_t arbiter, void (*
 #endif
 
 /* Try to submit just one data request, in case the request can be processed
- * immediatly, return 0, if there is still a dependency that is not compatible
+ * immediately, return 0, if there is still a dependency that is not compatible
  * with the current mode, the request is put in the per-handle list of
  * "requesters", and this function returns 1. */
 #ifdef LOCK_OR_DELEGATE
@@ -286,7 +288,7 @@ unsigned _starpu_attempt_to_submit_arbitered_data_request(unsigned request_from_
 		while (cpt < STARPU_SPIN_MAXTRY && _starpu_spin_trylock(&handle->header_lock))
 		{
 			cpt++;
-			_starpu_datawizard_progress(0);
+			_starpu_datawizard_progress(_STARPU_DATAWIZARD_DO_NOT_ALLOC);
 		}
 		if (cpt == STARPU_SPIN_MAXTRY)
 			_starpu_spin_lock(&handle->header_lock);
@@ -297,7 +299,7 @@ unsigned _starpu_attempt_to_submit_arbitered_data_request(unsigned request_from_
 	}
 
 	/* If there is currently nobody accessing the piece of data, or it's
-	 * not another writter and if this is the same type of access as the
+	 * not another writer and if this is the same type of access as the
 	 * current one, we can proceed. */
 	unsigned put_in_list = 1;
 
@@ -415,25 +417,36 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 
 	const unsigned start_buf_arbiter = buf;
 	unsigned idx_buf_arbiter;
+	int idx_buf_arbiterdup;
 	unsigned all_arbiter_available = 1;
 
 	starpu_data_handle_t handle;
 	enum starpu_data_access_mode mode;
+	int node;
 
 	for (idx_buf_arbiter = start_buf_arbiter; idx_buf_arbiter < nbuffers; idx_buf_arbiter++)
 	{
 		handle = descrs[idx_buf_arbiter].handle;
 		mode = descrs[idx_buf_arbiter].mode & ~STARPU_COMMUTE;
+		node = descrs[idx_buf_arbiter].orig_node;
 
 		mode = _starpu_arbiter_filter_modes(mode);
 
 		STARPU_ASSERT_MSG(!(mode & STARPU_REDUX), "REDUX with arbiter is not implemented\n");
 
-		if (idx_buf_arbiter && (descrs[idx_buf_arbiter-1].handle == handle))
-			/* We have already requested this data, skip it. This
-			 * depends on ordering putting writes before reads, see
-			 * _starpu_compar_handles.  */
-			continue;
+		for (idx_buf_arbiterdup = (int) idx_buf_arbiter-1; idx_buf_arbiterdup >= 0; idx_buf_arbiterdup--)
+		{
+			starpu_data_handle_t handle_dup = descrs[idx_buf_arbiterdup].handle;
+			int node_dup = descrs[idx_buf_arbiterdup].orig_node;
+			if (handle_dup == handle && node_dup == node)
+				/* We have already requested this data, skip it. This
+				 * depends on ordering putting writes before reads, see
+				 * _starpu_compar_handles.  */
+				goto next;
+			if (!_starpu_handles_same_root(handle_dup, handle))
+				/* We are not checking within the same parent any more, no need to continue checking other handles */
+				break;
+		}
 
 		if (handle->arbiter != arbiter)
 		{
@@ -459,6 +472,8 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 			all_arbiter_available = 0;
 			break;
 		}
+	next:
+		;
 	}
 	if (all_arbiter_available == 0)
 	{
@@ -482,16 +497,26 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 
 		/* and cancel all taken */
 		unsigned idx_buf_cancel;
+		unsigned idx_buf_canceldup;
 		for (idx_buf_cancel = start_buf_arbiter; idx_buf_cancel < idx_buf_arbiter ; idx_buf_cancel++)
 		{
 			starpu_data_handle_t cancel_handle = descrs[idx_buf_cancel].handle;
-
-			if (idx_buf_cancel && (descrs[idx_buf_cancel-1].handle == cancel_handle))
-				continue;
+			int cancel_node = descrs[idx_buf_cancel].orig_node;
 			if (cancel_handle->arbiter != arbiter)
 				/* Will have to process another arbiter, will do that later */
 				break;
 
+			for (idx_buf_canceldup = idx_buf_cancel+1; idx_buf_canceldup < idx_buf_arbiter; idx_buf_canceldup++)
+			{
+				starpu_data_handle_t handle_dup = descrs[idx_buf_canceldup].handle;
+				int node_dup = descrs[idx_buf_canceldup].orig_node;
+				if (handle_dup == cancel_handle && node_dup == cancel_node)
+					goto next2;
+				if (!_starpu_handles_same_root(handle_dup, cancel_handle))
+					/* We are not checking within the same parent any more, no need to continue checking other handles */
+					break;
+			}
+
 			_starpu_spin_lock(&cancel_handle->header_lock);
 			/* reset the counter because finally we do not take the data */
 			STARPU_ASSERT(cancel_handle->refcnt >= 1);
@@ -500,6 +525,8 @@ void _starpu_submit_job_enforce_arbitered_deps(struct _starpu_job *j, unsigned b
 			cancel_handle->busy_count--;
 			if (!_starpu_data_check_not_busy(cancel_handle))
 				_starpu_spin_unlock(&cancel_handle->header_lock);
+		next2:
+			;
 		}
 
 #ifndef LOCK_OR_DELEGATE
@@ -533,7 +560,7 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 }
 void ___starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 #else // LOCK_OR_DELEGATE
-void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
+void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle, enum starpu_data_access_mode down_to_mode)
 #endif
 {
 	starpu_arbiter_t arbiter = handle->arbiter;
@@ -546,10 +573,21 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 	{
 		/* No waiter, just remove our reference */
 		_starpu_spin_lock(&handle->header_lock);
-		STARPU_ASSERT(handle->refcnt > 0);
-		handle->refcnt--;
-		STARPU_ASSERT(handle->busy_count > 0);
-		handle->busy_count--;
+		if (down_to_mode == STARPU_NONE)
+		{
+			STARPU_ASSERT(handle->refcnt > 0);
+			handle->refcnt--;
+			STARPU_ASSERT(handle->busy_count > 0);
+			handle->busy_count--;
+		}
+		else
+		{
+			/* Downgrade from W or RW down to R, keeping the same reference,
+			 * but thus allowing other readers without allowing writers.  */
+			STARPU_ASSERT(down_to_mode == STARPU_R &&
+				      handle->current_mode == STARPU_W);
+			handle->current_mode = down_to_mode;
+		}
 #ifndef LOCK_OR_DELEGATE
 		STARPU_PTHREAD_MUTEX_UNLOCK(&arbiter->mutex);
 #endif
@@ -562,10 +600,21 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 
 	/* There is a waiter, remove our reference */
 	_starpu_spin_lock(&handle->header_lock);
-	STARPU_ASSERT(handle->refcnt > 0);
-	handle->refcnt--;
-	STARPU_ASSERT(handle->busy_count > 0);
-	handle->busy_count--;
+	if (down_to_mode == STARPU_NONE)
+	{
+		STARPU_ASSERT(handle->refcnt > 0);
+		handle->refcnt--;
+		STARPU_ASSERT(handle->busy_count > 0);
+		handle->busy_count--;
+	}
+	else
+	{
+		/* Downgrade from W or RW down to R, keeping the same reference,
+		 * but thus allowing other readers without allowing writers.  */
+		STARPU_ASSERT(down_to_mode == STARPU_R &&
+			      handle->current_mode == STARPU_W);
+		handle->current_mode = down_to_mode;
+	}
 	/* There should be at least one busy_count reference for the waiter
 	 * (thus we don't risk to see the handle disappear below) */
 	STARPU_ASSERT(handle->busy_count > 0);
@@ -626,9 +675,11 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 		unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(j->task);
 
 		unsigned idx_buf_arbiter;
+		int idx_buf_arbiterdup;
 		unsigned all_arbiter_available = 1;
 		starpu_data_handle_t handle_arbiter;
 		enum starpu_data_access_mode mode;
+		int node_arbiter;
 
 		unsigned start_buf_arbiter = r->buffer_index;
 		struct _starpu_data_descr *descrs = _STARPU_JOB_GET_ORDERED_BUFFERS(j);
@@ -636,15 +687,26 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 		for (idx_buf_arbiter = start_buf_arbiter; idx_buf_arbiter < nbuffers; idx_buf_arbiter++)
 		{
 			handle_arbiter = descrs[idx_buf_arbiter].handle;
-			if (idx_buf_arbiter && (descrs[idx_buf_arbiter-1].handle == handle_arbiter))
-				/* We have already requested this data, skip it. This
-				 * depends on ordering putting writes before reads, see
-				 * _starpu_compar_handles.  */
-				continue;
+			node_arbiter = descrs[idx_buf_arbiter].orig_node;
+
 			if (handle_arbiter->arbiter != arbiter)
 				/* Will have to process another arbiter, will do that later */
 				break;
 
+			for (idx_buf_arbiterdup = (int) idx_buf_arbiter-1; idx_buf_arbiterdup >= 0; idx_buf_arbiterdup--)
+			{
+				starpu_data_handle_t handle_dup = descrs[idx_buf_arbiterdup].handle;
+				int node_dup = descrs[idx_buf_arbiterdup].orig_node;
+				if (handle_dup == handle_arbiter && node_dup == node_arbiter)
+					/* We have already requested this data, skip it. This
+					 * depends on ordering putting writes before reads, see
+					 * _starpu_compar_handles.  */
+					goto next;
+				if (!_starpu_handles_same_root(handle_dup, handle_arbiter))
+					/* We are not checking within the same parent any more, no need to continue checking other handles */
+					break;
+			}
+
 			mode = descrs[idx_buf_arbiter].mode;
 			mode = _starpu_arbiter_filter_modes(mode);
 
@@ -662,6 +724,8 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 			handle_arbiter->busy_count++;
 			handle_arbiter->current_mode = mode;
 			_starpu_spin_unlock(&handle_arbiter->header_lock);
+		next:
+			;
 		}
 
 		if (all_arbiter_available)
@@ -711,13 +775,23 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 
 			/* and revert the mark */
 			unsigned idx_buf_cancel;
+			unsigned idx_buf_canceldup;
 			for (idx_buf_cancel = start_buf_arbiter; idx_buf_cancel < idx_buf_arbiter ; idx_buf_cancel++)
 			{
 				starpu_data_handle_t cancel_handle = descrs[idx_buf_cancel].handle;
-				if (idx_buf_cancel && (descrs[idx_buf_cancel-1].handle == cancel_handle))
-					continue;
+				int cancel_node = descrs[idx_buf_cancel].orig_node;
 				if (cancel_handle->arbiter != arbiter)
 					break;
+				for (idx_buf_canceldup = idx_buf_cancel+1; idx_buf_canceldup < idx_buf_arbiter; idx_buf_canceldup++)
+				{
+					starpu_data_handle_t handle_dup = descrs[idx_buf_canceldup].handle;
+					int node_dup = descrs[idx_buf_canceldup].orig_node;
+					if (handle_dup == cancel_handle && node_dup == cancel_node)
+						goto next2;
+					if (!_starpu_handles_same_root(handle_dup, cancel_handle))
+						/* We are not checking within the same parent any more, no need to continue checking other handles */
+						break;
+				}
 				_starpu_spin_lock(&cancel_handle->header_lock);
 				STARPU_ASSERT(cancel_handle->refcnt >= 1);
 				cancel_handle->refcnt--;
@@ -725,6 +799,8 @@ void _starpu_notify_arbitered_dependencies(starpu_data_handle_t handle)
 				cancel_handle->busy_count--;
 				if (!_starpu_data_check_not_busy(cancel_handle))
 					_starpu_spin_unlock(&cancel_handle->header_lock);
+			next2:
+				;
 			}
 		}
 	}
