@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2013  Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,11 +20,6 @@
 
 /** @file */
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
-
 /* Note: when changing something here, update the include list in configure.ac
  * in the part that tries to enable stdc++11 */
 #ifdef STARPU_SIMGRID
@@ -46,8 +41,20 @@ extern "C"
 #ifdef STARPU_HAVE_SIMGRID_HOST_H
 #include <simgrid/host.h>
 #endif
+#if defined(HAVE_SIMGRID_SIMDAG_H) && (SIMGRID_VERSION >= 31300)
+#include <simgrid/simdag.h>
+#endif
 
 #include <xbt/xbt_os_time.h>
+#endif
+
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
+#ifdef STARPU_SIMGRID
+#pragma GCC visibility push(hidden)
 
 struct _starpu_pthread_args
 {
@@ -55,6 +62,21 @@ struct _starpu_pthread_args
 	void *arg;
 };
 
+#if (SIMGRID_VERSION >= 32600)
+typedef void _starpu_simgrid_main_ret;
+#define _STARPU_SIMGRID_MAIN_RETURN do { } while (0)
+#else
+typedef int _starpu_simgrid_main_ret;
+#define _STARPU_SIMGRID_MAIN_RETURN return 0
+#endif
+#if (SIMGRID_VERSION >= 31500) && (SIMGRID_VERSION != 31559)
+typedef sg_link_t starpu_sg_link_t;
+#else
+typedef SD_link_t starpu_sg_link_t;
+#endif
+_starpu_simgrid_main_ret
+_starpu_simgrid_thread_start(int argc, char *argv[]);
+
 #define MAX_TSD 16
 
 #define STARPU_MPI_AS_PREFIX "StarPU-MPI"
@@ -64,21 +86,22 @@ void _starpu_start_simgrid(int *argc, char **argv);
 
 void _starpu_simgrid_init_early(int *argc, char ***argv);
 void _starpu_simgrid_init(void);
+void _starpu_simgrid_cpp_init(void);
 void _starpu_simgrid_deinit(void);
 void _starpu_simgrid_deinit_late(void);
 void _starpu_simgrid_actor_setup(void);
 void _starpu_simgrid_wait_tasks(int workerid);
 struct _starpu_job;
-void _starpu_simgrid_submit_job(int workerid, struct _starpu_job *job, struct starpu_perfmodel_arch* perf_arch, double length, unsigned *finished);
+void _starpu_simgrid_submit_job(int workerid, int sched_ctx_id, struct _starpu_job *job, struct starpu_perfmodel_arch* perf_arch, double length, double energy, unsigned *finished);
 struct _starpu_data_request;
 int _starpu_simgrid_transfer(size_t size, unsigned src_node, unsigned dst_node, struct _starpu_data_request *req);
-union _starpu_async_channel_event;
-int _starpu_simgrid_wait_transfer_event(union _starpu_async_channel_event *event);
-int _starpu_simgrid_test_transfer_event(union _starpu_async_channel_event *event);
+int _starpu_simgrid_wait_transfer_event(void *event);
+int _starpu_simgrid_test_transfer_event(void *event);
 void _starpu_simgrid_sync_gpus(void);
 /** Return the number of hosts prefixed by PREFIX */
 int _starpu_simgrid_get_nbhosts(const char *prefix);
 unsigned long long _starpu_simgrid_get_memsize(const char *prefix, unsigned devid);
+const char *_starpu_simgrid_get_devname(const char *prefix, unsigned devid);
 starpu_sg_host_t _starpu_simgrid_get_host_by_name(const char *name);
 starpu_sg_host_t _starpu_simgrid_get_memnode_host(unsigned node);
 struct _starpu_worker;
@@ -97,17 +120,25 @@ int _starpu_mpi_simgrid_init(int argc, char *argv[]);
 extern starpu_pthread_queue_t _starpu_simgrid_transfer_queue[STARPU_MAXNODES];
 extern starpu_pthread_queue_t _starpu_simgrid_task_queue[STARPU_NMAXWORKERS];
 
-#define _starpu_simgrid_cuda_malloc_cost() starpu_get_env_number_default("STARPU_SIMGRID_CUDA_MALLOC_COST", 1)
-#define _starpu_simgrid_queue_malloc_cost() starpu_get_env_number_default("STARPU_SIMGRID_QUEUE_MALLOC_COST", 1)
-#define _starpu_simgrid_task_submit_cost() starpu_get_env_number_default("STARPU_SIMGRID_TASK_SUBMIT_COST", 1)
-#define _starpu_simgrid_fetching_input_cost() starpu_get_env_number_default("STARPU_SIMGRID_FETCHING_INPUT_COST", 1)
-#define _starpu_simgrid_sched_cost() starpu_get_env_number_default("STARPU_SIMGRID_SCHED_COST", 0)
+#ifdef STARPU_HAVE_S4U_ON_TIME_ADVANCE_CB
+extern starpu_pthread_mutex_t _starpu_simgrid_time_advance_mutex;
+extern starpu_pthread_cond_t _starpu_simgrid_time_advance_cond;
+#endif
+
+#define _starpu_simgrid_cuda_malloc_cost() starpu_getenv_number_default("STARPU_SIMGRID_CUDA_MALLOC_COST", 1)
+#define _starpu_simgrid_cuda_queue_cost() starpu_getenv_number_default("STARPU_SIMGRID_CUDA_QUEUE_COST", 1)
+#define _starpu_simgrid_task_submit_cost() starpu_getenv_number_default("STARPU_SIMGRID_TASK_SUBMIT_COST", 1)
+#define _starpu_simgrid_task_push_cost() starpu_getenv_number_default("STARPU_SIMGRID_TASK_PUSH_COST", 1)
+#define _starpu_simgrid_fetching_input_cost() starpu_getenv_number_default("STARPU_SIMGRID_FETCHING_INPUT_COST", 1)
+#define _starpu_simgrid_sched_cost() starpu_getenv_number_default("STARPU_SIMGRID_SCHED_COST", 0)
 
 /** Called at initialization to count how many GPUs are interfering with each
  * bus */
 void _starpu_simgrid_count_ngpus(void);
 
-void _starpu_simgrid_xbt_thread_create(const char *name, void_f_pvoid_t code,
+extern size_t _starpu_default_stack_size;
+void _starpu_simgrid_set_stack_size(size_t stack_size);
+void _starpu_simgrid_xbt_thread_create(const char *name, starpu_pthread_attr_t *attr, void_f_pvoid_t code,
 				       void *param);
 
 #define _SIMGRID_TIMER_BEGIN(cond)			\
@@ -125,6 +156,8 @@ void _starpu_simgrid_xbt_thread_create(const char *name, void_f_pvoid_t code,
 		}	\
 	}
 
+#pragma GCC visibility pop
+
 #else // !STARPU_SIMGRID
 #define _SIMGRID_TIMER_BEGIN(cond) {
 #define _SIMGRID_TIMER_END }
