@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2013       Simon Archipoff
- * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2008-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2013  Thibaut Lambert
+ * Copyright (C) 2013-2013  Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,20 +22,17 @@
 #include <core/workers.h>
 #include <core/sched_ctx.h>
 #include <core/sched_policy.h>
-#include <sched_policies/fifo_queues.h>
 #include <core/debug.h>
 #include <core/task.h>
 
 static int _random_push_task(struct starpu_task *task, unsigned prio)
 {
 	/* find the queue */
-
-
 	double alpha_sum = 0.0;
 
 	unsigned sched_ctx_id = task->sched_ctx;
 	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
-        int worker;
+	int worker;
 	int worker_arr[STARPU_NMAXWORKERS];
 	double speedup_arr[STARPU_NMAXWORKERS];
 	int size = 0;
@@ -44,7 +41,7 @@ static int _random_push_task(struct starpu_task *task, unsigned prio)
 	workers->init_iterator(workers, &it);
 	while(workers->has_next(workers, &it))
 	{
-                worker = workers->get_next(workers, &it);
+		worker = workers->get_next(workers, &it);
 		unsigned impl;
 		if(starpu_worker_can_execute_task_first_impl(worker, task, &impl))
 		{
@@ -68,16 +65,16 @@ static int _random_push_task(struct starpu_task *task, unsigned prio)
 	int i;
 	for(i = 0; i < size; i++)
 	{
-                worker = worker_arr[i];
+		worker = worker_arr[i];
 		double worker_alpha = speedup_arr[i];
-		
+
 		if (alpha + worker_alpha >= random)
 		{
 			/* we found the worker */
 			selected = worker;
 			break;
 		}
-		
+
 		alpha += worker_alpha;
 	}
 	STARPU_AYU_ADDTOTASKQUEUE(starpu_task_get_job_id(task), selected);
@@ -87,7 +84,7 @@ static int _random_push_task(struct starpu_task *task, unsigned prio)
 
 static int random_push_task(struct starpu_task *task)
 {
-        return _random_push_task(task, !!task->priority);
+	return _random_push_task(task, !!task->priority);
 }
 
 static void initialize_random_policy(unsigned sched_ctx_id)
@@ -111,7 +108,6 @@ struct starpu_sched_policy _starpu_sched_random_policy =
 	.pop_task = NULL,
 	.pre_exec_hook = NULL,
 	.post_exec_hook = NULL,
-	.pop_every_task = NULL,
 	.policy_name = "random",
 	.policy_description = "weighted random based on worker overall performance",
 	.worker_type = STARPU_WORKER_LIST,
