@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2020-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2020-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -757,7 +757,9 @@ static PyObject* free_perfmodel(PyObject *self, PyObject *args)
 	struct starpu_perfmodel *perf=PyCapsule_GetPointer(perfmodel, "Perf");
 
 	Py_BEGIN_ALLOW_THREADS;
+#ifndef STARPU_SIMGRID
 	starpu_save_history_based_model(perf);
+#endif
 	//starpu_perfmodel_unload_model(perf);
 	starpu_perfmodel_deinit(perf);
 	Py_END_ALLOW_THREADS;
@@ -769,6 +771,7 @@ static PyObject* free_perfmodel(PyObject *self, PyObject *args)
 	return Py_None;
 }
 
+#ifndef STARPU_SIMGRID
 static PyObject* starpu_save_history_based_model_wrapper(PyObject *self, PyObject *args)
 {
 	(void)self;
@@ -809,6 +812,7 @@ static PyObject* starpu_save_history_based_model_wrapper(PyObject *self, PyObjec
 	Py_INCREF(Py_None);
 	return Py_None;
 }
+#endif
 
 /*****************************Wrappers of StarPU methods****************************/
 /*wrapper submit method*/
@@ -853,6 +857,7 @@ static PyObject* starpu_task_submit_wrapper(PyObject *self, PyObject *args)
 	starpu_codelet_init(func_cl);
 	func_cl->cpu_funcs[0] = &starpupy_codelet_func;
 	func_cl->cpu_funcs_name[0] = "starpupy_codelet_func";
+	func_cl->flags = STARPU_CODELET_SIMGRID_EXECUTE;
 
 	int h_index = 0, h_flag = 0;
 	int nbuffer = 0;
@@ -1538,8 +1543,23 @@ static void new_inter(void* arg)
 	STARPU_ASSERT(state == PyGILState_UNLOCKED);
 	orig_thread_states[workerid] = PyThreadState_GET();
 
-	/* TODO: Use Py_NewInterpreterEx when https://peps.nogil.dev/pep-0684/ gets released */
-	new_thread_state = Py_NewInterpreter();
+	if (starpu_getenv_number_default("STARPUPY_OWN_GIL", 0))
+	{
+#ifdef PyInterpreterConfig_OWN_GIL
+		/* https://peps.nogil.dev/pep-0684/ */
+		PyInterpreterConfig config = {
+		    .check_multi_interp_extensions = 1,
+		    .gil = PyInterpreterConfig_OWN_GIL,
+		};
+		Py_NewInterpreterFromConfig(&new_thread_state, &config);
+#else
+		fprintf(stderr, "STARPUPY_OWN_GIL is only supported starting from python 3.12\n");
+		exit(1);
+#endif
+	}
+	else
+		new_thread_state = Py_NewInterpreter();
+
 	PyThreadState_Swap(new_thread_state);
 	new_thread_states[workerid] = new_thread_state;
 	PyEval_SaveThread(); // releases the GIL
@@ -1755,6 +1775,10 @@ static PyObject* starpu_set_ncpu(PyObject *self, PyObject *args)
 
 	starpu_shutdown();
 
+	if (starpu_getenv("STARPU_NCPU") ||
+	    starpu_getenv("STARPU_NCPUS"))
+		fprintf(stderr, "warning: starpupy.set_ncpu is ineffective when the STARPU_NCPU or STARPU_NCPUS environment variable is defined");
+
 	int ret;
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
@@ -1793,13 +1817,15 @@ static PyMethodDef starpupyMethods[] =
 	{"resume", starpu_resume_wrapper, METH_VARARGS, "resume the workers polling for new tasks"}, /*resume method*/
 	{"init_perfmodel", init_perfmodel, METH_VARARGS, "initialize struct starpu_perfmodel"}, /*initialize perfmodel*/
 	{"free_perfmodel", free_perfmodel, METH_VARARGS, "free struct starpu_perfmodel"}, /*free perfmodel*/
+#ifndef STARPU_SIMGRID
 	{"save_history_based_model", starpu_save_history_based_model_wrapper, METH_VARARGS, "save the performance model"}, /*save the performance model*/
+#endif
 	{"sched_get_min_priority", starpu_sched_get_min_priority_wrapper, METH_VARARGS, "get the number of min priority"}, /*get the number of min priority*/
 	{"sched_get_max_priority", starpu_sched_get_max_priority_wrapper, METH_VARARGS, "get the number of max priority"}, /*get the number of max priority*/
 	{"task_nsubmitted", starpu_task_nsubmitted_wrapper, METH_VARARGS, "get the number of submitted tasks which have not completed yet"}, /*get the number of submitted tasks which have not completed yet*/
 	{"shutdown", starpu_shutdown_wrapper, METH_VARARGS, "shutdown starpu"}, /*shutdown starpu*/
-	{"starpupy_data_register", starpupy_data_register_wrapper, METH_VARARGS, "register PyObject in a handle"}, /*reigster PyObject in a handle*/
-	{"starpupy_numpy_register", starpupy_numpy_register_wrapper, METH_VARARGS, "register empty Numpy array in a handle"}, /*reigster PyObject in a handle*/
+	{"starpupy_data_register", starpupy_data_register_wrapper, METH_VARARGS, "register PyObject in a handle"}, /*register PyObject in a handle*/
+	{"starpupy_numpy_register", starpupy_numpy_register_wrapper, METH_VARARGS, "register empty Numpy array in a handle"}, /*register PyObject in a handle*/
 	{"starpupy_get_object", starpupy_get_object_wrapper, METH_VARARGS, "get PyObject from handle"}, /*get PyObject from handle*/
 	{"starpupy_acquire_handle", starpupy_acquire_handle_wrapper, METH_VARARGS, "acquire handle"}, /*acquire handle*/
 	{"starpupy_release_handle", starpupy_release_handle_wrapper, METH_VARARGS, "release handle"}, /*release handle*/
@@ -1892,6 +1918,14 @@ PyMODINIT_FUNC PyInit_starpupy(void)
 	PyEval_InitThreads();
 #endif
 
+#if defined(STARPU_USE_MPI_MASTER_SLAVE)
+	active_multi_interpreter = 1;
+#else
+	if (starpu_getenv_number_default("STARPUPY_MULTI_INTERPRETER", 0)
+		|| starpu_getenv_number("STARPU_TCPIP_MS_SLAVES") > 0)
+		active_multi_interpreter = 1;
+#endif
+
 	main_thread = pthread_self();
 
 	/*python asyncio import*/
@@ -1904,26 +1938,32 @@ PyMODINIT_FUNC PyInit_starpupy(void)
 	}
 
 	/*cloudpickle import*/
-	cloudpickle_module = PyImport_ImportModule("cloudpickle");
-	if (cloudpickle_module == NULL)
+	if (active_multi_interpreter)
 	{
-		PyErr_Format(PyExc_RuntimeError, "can't find cloudpickle module");
-		starpupyFree(NULL);
-		return NULL;
+		cloudpickle_module = PyImport_ImportModule("cloudpickle");
+		if (cloudpickle_module == NULL)
+		{
+			PyErr_Format(PyExc_RuntimeError, "can't find cloudpickle module");
+			starpupyFree(NULL);
+			return NULL;
+		}
+		/*dumps method*/
+		dumps = PyObject_GetAttrString(cloudpickle_module, "dumps");
 	}
-	/*dumps method*/
-	dumps = PyObject_GetAttrString(cloudpickle_module, "dumps");
 
 	/*pickle import*/
-	pickle_module = PyImport_ImportModule("pickle");
-	if (pickle_module == NULL)
+	if (active_multi_interpreter)
 	{
-		PyErr_Format(PyExc_RuntimeError, "can't find pickle module");
-		starpupyFree(NULL);
-		return NULL;
+		pickle_module = PyImport_ImportModule("pickle");
+		if (pickle_module == NULL)
+		{
+			PyErr_Format(PyExc_RuntimeError, "can't find pickle module");
+			starpupyFree(NULL);
+			return NULL;
+		}
+		/*loads method*/
+		loads = PyObject_GetAttrString(pickle_module, "loads");
 	}
-	/*loads method*/
-	loads = PyObject_GetAttrString(pickle_module, "loads");
 
 	/*starpu import*/
 	starpu_module = PyImport_ImportModule("starpu");
@@ -2029,15 +2069,6 @@ PyMODINIT_FUNC PyInit_starpupy(void)
 		return NULL;
 	}
 
-
-#if defined(STARPU_USE_MPI_MASTER_SLAVE)
-	active_multi_interpreter = 1;
-#else
-	if (starpu_getenv_number_default("STARPUPY_MULTI_INTERPRETER", 0)
-		|| starpu_getenv_number("STARPU_TCPIP_MS_SLAVES") > 0)
-		active_multi_interpreter = 1;
-#endif
-
 	/*module import multi-phase initialization*/
 	return PyModuleDef_Init(&starpupymodule);
 }
