@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2013  Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,9 +28,9 @@
  */
 
 #ifdef STARPU_QUICK_CHECK
-#  define NLOOPS		100
+static unsigned nloops=100;
 #else
-#  define NLOOPS		1000
+static unsigned nloops=1000;
 #endif
 #define VECTORSIZE	1024
 
@@ -57,6 +57,7 @@ static void cuda_memset_codelet(void *descr[], void *arg)
 static void opencl_memset_codelet(void *buffers[], void *args)
 {
 	(void) args;
+	STARPU_SKIP_IF_VALGRIND;
 
 	cl_command_queue queue;
 	int id = starpu_worker_get_id_check();
@@ -69,15 +70,17 @@ static void opencl_memset_codelet(void *buffers[], void *args)
 	STARPU_ASSERT(v != NULL);
 	memset(v, 42, length);
 
-	clEnqueueWriteBuffer(queue,
-			     buffer,
-			     CL_FALSE,
-			     0,      /* offset */
-			     length, /* sizeof (char) */
-			     v,
-			     0,      /* num_events_in_wait_list */
-			     NULL,   /* event_wait_list */
-			     NULL    /* event */);
+	cl_int err;
+	err = clEnqueueWriteBuffer(queue,
+				   buffer,
+				   CL_FALSE,
+				   0,      /* offset */
+				   length, /* sizeof (char) */
+				   v,
+				   0,      /* num_events_in_wait_list */
+				   NULL,   /* event_wait_list */
+				   NULL    /* event */);
+	if (STARPU_UNLIKELY(err != CL_SUCCESS)) STARPU_OPENCL_REPORT_ERROR(err);
 }
 #endif /* !STARPU_USE_OPENCL */
 
@@ -148,13 +151,21 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	if(starpu_cpu_worker_get_count() == 0) return STARPU_TEST_SKIPPED;
+	if(starpu_cpu_worker_get_count() == 0)
+	{
+		starpu_shutdown();
+		return STARPU_TEST_SKIPPED;
+	}
 
-	/* The buffer should never be explicitely allocated */
+#ifdef STARPU_HAVE_VALGRIND_H
+	if(RUNNING_ON_VALGRIND) nloops = 2;
+#endif
+
+	/* The buffer should never be explicitly allocated */
 	starpu_vector_data_register(&v_handle, (uint32_t)-1, (uintptr_t)NULL, VECTORSIZE, sizeof(char));
 
 	unsigned loop;
-	for (loop = 0; loop < NLOOPS; loop++)
+	for (loop = 0; loop < nloops; loop++)
 	{
 		struct starpu_task *memset_task;
 		struct starpu_task *check_content_task;
@@ -186,7 +197,7 @@ int main(int argc, char **argv)
 		starpu_data_invalidate(v_handle);
 	}
 
-	for (loop = 0; loop < NLOOPS; loop++)
+	for (loop = 0; loop < nloops; loop++)
 	{
 		struct starpu_task *memset_task;
 		struct starpu_task *check_content_task;
