@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -197,8 +197,10 @@ int main(int argc, char **argv)
 
 	ret = starpu_malloc((void **)&v, VECTORSIZE*sizeof(unsigned));
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
+	memset(v, 0, VECTORSIZE*sizeof(unsigned));
 	ret = starpu_malloc((void **)&v2, VECTORSIZE*sizeof(unsigned));
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_malloc");
+	memset(v2, 0, VECTORSIZE*sizeof(unsigned));
 
 	starpu_vector_data_register(&v_handle, STARPU_MAIN_RAM, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
 	starpu_vector_data_register(&v_handle2, STARPU_MAIN_RAM, (uintptr_t)v2, VECTORSIZE, sizeof(unsigned));
@@ -249,8 +251,8 @@ int main(int argc, char **argv)
 
 	starpu_data_unregister(v_handle);
 	starpu_data_unregister(v_handle2);
-	starpu_free(v);
-	starpu_free(v2);
+	starpu_free_noflag(v, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(v2, VECTORSIZE*sizeof(unsigned));
 	starpu_shutdown();
 
 	return EXIT_SUCCESS;
@@ -258,8 +260,8 @@ int main(int argc, char **argv)
 enodev:
 	starpu_data_unregister(v_handle);
 	starpu_data_unregister(v_handle2);
-	starpu_free(v);
-	starpu_free(v2);
+	starpu_free_noflag(v, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(v2, VECTORSIZE*sizeof(unsigned));
 	starpu_shutdown();
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
