@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,8 +16,7 @@
 
 #include <starpu.h>
 #include <common/thread.h>
-
-#include "increment_codelet.h"
+#include "../variable/increment.h"
 #include "../helper.h"
 
 /*
@@ -93,23 +92,25 @@ int main(int argc, char **argv)
 	starpu_malloc((void**)&check_cnt, sizeof(*check_cnt));
 	*check_cnt = 0;
 
+	increment_load_opencl();
+
 	starpu_data_handle_t check_data;
 	starpu_variable_data_register(&check_data, STARPU_MAIN_RAM, (uintptr_t)check_cnt, sizeof(*check_cnt));
 
 	starpu_task_init(&taskA);
-	taskA.cl = &increment_codelet;
+	taskA.cl = &increment_cl;
 	taskA.handles[0] = check_data;
 
 	starpu_task_init(&taskB);
-	taskB.cl = &increment_codelet;
+	taskB.cl = &increment_cl;
 	taskB.handles[0] = check_data;
 
 	starpu_task_init(&taskC);
-	taskC.cl = &increment_codelet;
+	taskC.cl = &increment_cl;
 	taskC.handles[0] = check_data;
 
 	starpu_task_init(&taskD);
-	taskD.cl = &increment_codelet;
+	taskD.cl = &increment_cl;
 	taskD.callback_func = callback_task_D;
 	taskD.handles[0] = check_data;
 
@@ -135,7 +136,7 @@ int main(int argc, char **argv)
 
 	STARPU_ASSERT(*check_cnt == (4*loop_cnt));
 
-	starpu_free(check_cnt);
+	starpu_free_noflag(check_cnt, sizeof(*check_cnt));
 	starpu_data_unregister(check_data);
 
 	starpu_task_wait_for_all();
@@ -144,6 +145,8 @@ int main(int argc, char **argv)
 	starpu_task_clean(&taskC);
 	starpu_task_clean(&taskD);
 
+	increment_unload_opencl();
+
 	starpu_shutdown();
 
 	return EXIT_SUCCESS;
@@ -153,6 +156,7 @@ enodev:
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
 	starpu_data_unregister(check_data);
+	increment_unload_opencl();
 	starpu_shutdown();
 	return STARPU_TEST_SKIPPED;
 }
